﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace Voltmeter
{
    public partial class Form1 : Form
    {
        int timerCount = 0;
        IntPtr m_Handle = IntPtr.Zero;
        StringBuilder m_serial = new StringBuilder();
        uint m_pid = 0;

        byte[] m_write = new byte[IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE];                    
        byte[] m_read = new byte[IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE];

        int m_Line = 0;
        string m_FileName;
        TextDatei m_TextDatei = new TextDatei();
        

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Init dialog elements
            range.SelectedIndex = 0;
            timerMin.SelectedIndex = 0;
            timerSec.SelectedIndex = 0;
            timerMSec.SelectedIndex = 0;

            channel1.Text = "0.000 V";
            channel2.Text = "0.000 V";
            channel3.Text = "0.000 V";
            channel4.Text = "0.000 V";
            channel5.Text = "0.000 V";
            channel6.Text = "0.000 V";
            channel7.Text = "0.000 V";
            channel8.Text = "0.000 V";
        }

        /*Connect*/
        private void buttonConnect_Click(object sender, EventArgs e)
        {
            m_Handle = IntPtr.Zero;
            m_Handle = IOWarrior.Functions.IowKitOpenDevice();
            if (m_Handle != IntPtr.Zero)
            {
                uint numDevs = IOWarrior.Functions.IowKitGetNumDevs();

                //Device count must be 1!
                if (numDevs == 1)
                {
                    
                    m_pid = IOWarrior.Functions.IowKitGetProductId(m_Handle);

                    //Only IO-Warrior24 will work in this programm
                    if (m_pid == IOWarrior.Defines.IOWKIT_PID_IOW24)
                    {
                        //Read serialnumber from device
                        IOWarrior.Functions.IowKitGetSerialNumber(m_Handle, m_serial);

                        //Set timeout for preventing "death hang" in program
                        IOWarrior.Functions.IowKitSetTimeout(m_Handle, 1000);

                        //Type device infos
                        serialnumber.Text = m_serial.ToString();
                        status.Text = "Connected";
                        status.BackColor = Color.Green;
                        device.Text = "IO-Warrior24";

                        //Init IOW24 for I2C, take a look into datasheet for IO-Warrior24
                        m_write[0] = 0x01; //I2C Mode
                        m_write[1] = 0x01; //Enable
                        m_write[2] = 0x00; //Flags
                        m_write[3] = 0x00; //Timeout
                        IOWarrior.Functions.IowKitWrite(m_Handle, IOWarrior.Defines.IOW_PIPE_SPECIAL_MODE, m_write, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);

                        //Enable dialog elements
                        timerMin.Enabled = true;
                        timerSec.Enabled = true;
                        timerMSec.Enabled = true;
                        buttonStart.Enabled = true;
                        buttonDatei.Enabled = true;
                    }
                    else
                    {
                        status.Text = "Error wrong device";
                        status.BackColor = Color.Orange;
                        MessageBox.Show("This programm only works with the IO-Warrior24 or IO-Warrior24 DG.");
                    }
                }
                else
                {
                    status.Text = "Error too many devices";
                    status.BackColor = Color.Orange;
                    MessageBox.Show("Found more than one IO-Warrior device. Please disconnect all IO-Warrior wich will not used by this program.");
                }
            }
            else
            {
                status.Text = "Error no device";
                status.BackColor = Color.Orange;
                MessageBox.Show("No device found. Please connect an IO-Warrior24 or IO-Warrior24 DG.");
            }
        }

        /*Disconnect*/
        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            //Kill timer
            if (timer1.Enabled)
                timer1.Enabled = false;

            status.Text = "Disconnected";
            status.BackColor = Color.Red;
            IOWarrior.Functions.IowKitCloseDevice(m_Handle);      

            //Disable dialog elements
            timerMin.Enabled = false;
            timerSec.Enabled = false;
            timerMSec.Enabled = false;
            buttonStart.Enabled = false;
            buttonDatei.Enabled = false;
            checkRecord.Enabled = false;
            checkRecord.Checked = false;

            m_FileName = "";

            serialnumber.Text = "00000000";
            device.Text = "None";
            m_Handle = IntPtr.Zero;
        }

        private void buttomStart_Click(object sender, EventArgs e)
        {
            if (!timer1.Enabled)
            {
                //Calulate timer tick
                int minute = timerMin.SelectedIndex * 60000;
                int sekunde = timerSec.SelectedIndex * 1000;
                int msekunde = timerMSec.SelectedIndex * 50;
                int time = minute + sekunde + msekunde;

                //Set timertick
                timer1.Interval = time;

                //Start Timer if !0
                if (time > 0)
                {
                    timer1.Enabled = true;
                    buttonStart.Text = "Stopp";

                    //Disable dialog elements
                    timerMin.Enabled = false;
                    timerSec.Enabled = false;
                    timerMSec.Enabled = false;
                }
                else
                    MessageBox.Show("Timer is set to Zero. Please change timer settings.");
            }
            else
            {
                timer1.Enabled = false;
                buttonStart.Text = "Start";

                //Enable dialog elements
                timerMin.Enabled = true;
                timerSec.Enabled = true;
                timerMSec.Enabled = true;
             }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            timerCount++;
            uint ret;

            counter.Text = timerCount.ToString();
            int summ = 0;
            float[] ergebnisse = new float[8];
            string output = "";

            //Create Time
            DateTime dateToDisplay = new DateTime(System.DateTime.Now.Year, System.DateTime.Now.Month, System.DateTime.Now.Day, System.DateTime.Now.Hour, System.DateTime.Now.Minute, System.DateTime.Now.Second);
            string time = dateToDisplay.ToString("T", CultureInfo.CreateSpecificCulture("de-DE"));

            for (int ii = 0; ii < 8; ii++)
            {
                m_write[0] = 0x02; //I2C Write
                m_write[1] = 0xC2; //2Bytes Start & Stopp
                m_write[2] = 0x50; //Address
                m_write[3] = (byte)(0x88 | ii << 4);    //Channelindex, RGN 1 and BIP 0, normal operation for +/-10V
                ret = IOWarrior.Functions.IowKitWrite(m_Handle, IOWarrior.Defines.IOW_PIPE_SPECIAL_MODE, m_write, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);
                //Swallow ACK
                ret = IOWarrior.Functions.IowKitRead(m_Handle, IOWarrior.Defines.IOW_PIPE_SPECIAL_MODE, m_read, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);
         
                m_write[0] = 0x03; //I2C Write
                m_write[1] = 0x02; //2Bytes Start & Stopp
                m_write[2] = 0x51; //Address
               // m_write[3] = 0x8C | (ii << 4);   //8C   Start, Channel 0, RNG 1, BIP 1, PD1 0, PD0 0
                m_write[3] = (byte)(0x88 | ii << 4);   //8C   Start, Channel 0, RNG 1, BIP 0, PD1 0, PD0 0
                ret = IOWarrior.Functions.IowKitWrite(m_Handle, IOWarrior.Defines.IOW_PIPE_SPECIAL_MODE, m_write, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);
                //Read Data
                ret = IOWarrior.Functions.IowKitRead(m_Handle, 1, m_read, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);

                //Calculate voltage
                summ = (m_read[2] << 8) | m_read[3];
                summ = summ >> 4;
                ergebnisse[ii] = (float)((summ * 10.0) / 4096);
            }

            output += time + ";";

            //Show voltag in lable
            if (channel1.Enabled)
            {
                channel1.Text = ergebnisse[0].ToString("0.000 V");
                output += ergebnisse[0].ToString("0.000") + ";";
            }
            if (channel2.Enabled)
            {
                channel2.Text = ergebnisse[1].ToString("0.000 V");
                output += ergebnisse[1].ToString("0.000") + ";";
            }
            if (channel3.Enabled)
            {
                channel3.Text = ergebnisse[2].ToString("0.000 V");
                output += ergebnisse[2].ToString("0.000") + ";";
            }
            if (channel4.Enabled)
            {
                channel4.Text = ergebnisse[3].ToString("0.000 V");
                output += ergebnisse[3].ToString("0.000") + ";";
            }
            if (channel5.Enabled)
            {
                channel5.Text = ergebnisse[4].ToString("0.000 V");
                output += ergebnisse[4].ToString("0.000") + ";";
            }
            if (channel6.Enabled)
            {
                channel6.Text = ergebnisse[5].ToString("0.000 V");
                output += ergebnisse[5].ToString("0.000") + ";";
            }
            if (channel7.Enabled)
            {
                channel7.Text = ergebnisse[6].ToString("0.000 V");
                output += ergebnisse[6].ToString("0.000") + ";";
            }
            if (channel8.Enabled)
            {
                channel8.Text = ergebnisse[7].ToString("0.000 V");
                output += ergebnisse[7].ToString("0.000") + ";";
            }


            //Write into file
            if (checkRecord.Checked)
            {
                m_Line++;
               // DateTime dateToDisplay = new DateTime(System.DateTime.Now.Year, System.DateTime.Now.Month, System.DateTime.Now.Day, System.DateTime.Now.Hour, System.DateTime.Now.Minute, System.DateTime.Now.Second);
//                string time = dateToDisplay.ToString("T", CultureInfo.CreateSpecificCulture("de-DE"));

               // output = time + ";" + ergebnisse[0].ToString("0.000") + ";" + ergebnisse[1].ToString("0.000") + ";" + 
                 //        ergebnisse[2].ToString("0.000") + ";" + ergebnisse[3].ToString("0.000") + ";" + ergebnisse[4].ToString("0.000") + ";" + 
                   //      ergebnisse[5].ToString("0.000") + ";" + ergebnisse[6].ToString("0.000") + ";" + ergebnisse[7].ToString("0.000");

                m_TextDatei.WriteLine(m_FileName, m_Line, output, true);
            }            
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (m_Handle != IntPtr.Zero)
            {
                m_write[0] = 0x01; //I2C Mode
                m_write[1] = 0x00; //Enable
                m_write[2] = 0x00;
                m_write[3] = 0x00;
                IOWarrior.Functions.IowKitWrite(m_Handle, IOWarrior.Defines.IOW_PIPE_SPECIAL_MODE, m_write, IOWarrior.Defines.IOWKIT_SPECIAL_REPORT_SIZE);
            }

            IOWarrior.Functions.IowKitCloseDevice(m_Handle);
        }

        private void buttonDatei_Click(object sender, EventArgs e)
        {
            saveFileDialog1.DefaultExt = "txt";
            saveFileDialog1.Filter = "Text files (*.txt)|*.txt";

            //Open save dialog
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                m_FileName = saveFileDialog1.FileName;  //Names des Pfades
                m_TextDatei.WriteFile(m_FileName, "");

                file.Text = m_FileName.ToString();
                checkRecord.Enabled = true;
            }
        }

        //Enable / Disable Labelelelemt
        public void EnableChannel(System.Windows.Forms.Label title, System.Windows.Forms.Label input)
        {
            if (!title.Enabled)
            {
                title.Enabled = true;
                input.Enabled = true;
            }
            else
            {
                title.Enabled = false;
                input.Enabled = false;
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel1, channel1);
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel2, channel2); 
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel3, channel3);
        }

        private void checkBox4_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel4, channel4);
        }

        private void checkBox5_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel5, channel5);
        }

        private void checkBox6_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel6, channel6);
        }

        private void checkBox7_CheckedChanged(object sender, EventArgs e)
        {
            EnableChannel(labelChannel7, channel7);
        }

        private void checkBox8_CheckedChanged(object sender, EventArgs e)
        {
             EnableChannel(labelChannel8, channel8);
        }
    }
}
