unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvHidControllerClass, StdCtrls, ExtCtrls;

const
  cCodeMercsVID = $07C0;
  cIOW40PID = $1500;
  cIOW24PID = $1501;

type
  TIICReport = packed record
    ReportID: Byte;
    Bytes: array [0..6] of Byte;
  end;

  TForm1 = class(TForm)
    HidCtl: TJvHidDeviceController;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Image1: TImage;
    Kicker: TTimer;
    Label25: TLabel;
    procedure HidCtlDeviceChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure KickerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  public
    IOWIIC: TJvHIDDevice;
    Values: array [0..7] of TLabel;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Values[0] := Label1;
  Values[1] := Label2;
  Values[2] := Label3;
  Values[3] := Label4;
  Values[4] := Label5;
  Values[5] := Label6;
  Values[6] := Label7;
  Values[7] := Label8;
end;

procedure TForm1.HidCtlDeviceChange(Sender: TObject);
var
  I: Integer;
  Report: TIICReport;
  BytesWritten: DWORD;

  // all IOWarriors have IIC
  function FindIOWComplex(HidDev: TJvHidDevice): Boolean; stdcall;
  begin
    // IO-Warrior special mode interface match criteria
    Result := (HidDev.Attributes.VendorID = cCodeMercsVID) and
      ((HidDev.Attributes.ProductID = cIOW24PID) or
       (HidDev.Attributes.ProductID = cIOW40PID)) and
      (HidDev.Caps.OutputReportByteLength = 8);
  end;

begin
  // free IOW object if it has been unplugged
  if Assigned(IOWIIC) and not IOWIIC.IsPluggedIn then
  begin
    // disable reader timer
    Kicker.Enabled := False;
    FreeAndNil(IOWIIC);
  end;
  // search for an IOW if not already found
  if not Assigned(IOWIIC) then
    if HidCtl.CheckOutByCallback(IOWIIC, @FindIOWComplex) then
    begin
      // IOW found, enable IIC
      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $01;
      Report.Bytes[0] := $01; // enable IIC
      IOWIIC.WriteFile(Report, SizeOf(Report), BytesWritten);
      // enable reader timer
      Kicker.Enabled := True;
    end
    else
      for I := Low(Values) to High(Values) do
        // no IOW, show bogus value
        Values[I].Caption := '?,?? V';
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Report: TIICReport;
  BytesWritten: DWORD;
begin
  // if IOW still plugged when program terminates
  if Assigned(IOWIIC) then
  begin
    // disable IIC
    FillChar(Report, SizeOf(Report), 0);
    Report.ReportID := $01;
    // Report.Bytes[0] := $00;
    IOWIIC.WriteFile(Report, SizeOf(Report), BytesWritten);
  end;
end;

procedure TForm1.KickerTimer(Sender: TObject);
var
  I: Integer;
  Report: TIICReport;
  BytesWritten, BytesRead: DWORD;
  N: Integer;
begin
  if Assigned(IOWIIC) then
  begin
    for I := 0 to 7 do
    begin
      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $02; // ReportID IIC write request
      Report.Bytes[0] := $C2; // 2 bytes with IIC Start and Stop
      Report.Bytes[1] := $50; // MAXIM127 address byte = ADR 0, write
      // MAXIM127 control byte
      // START, I = Channel index, 0..10 V = RNG 1 and BIP 0, normal operation
      Report.Bytes[2] := $8C or (I shl 4);
      IOWIIC.WriteFile(Report, SizeOf(Report), BytesWritten);
      // swallow ACK report
      IOWIIC.ReadFile(Report, SizeOf(Report), BytesRead);

      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $03; // ReportID IIC read request
      Report.Bytes[0] := $02; // 2 bytes
      Report.Bytes[1] := $51; // MAXIM127 address byte = ADR 0, read
      // MAXIM127 control byte
      // START, I = Channel index, 0..10 V = RNG 1 and BIP 0, normal operation
      Report.Bytes[2] := $8C or (I shl 4);
      IOWIIC.WriteFile(Report, SizeOf(Report), BytesWritten);
      // in this case we cannot use the OnData event because we would lose
      // the index I which is not part of the answer
      IOWIIC.ReadFile(Report, SizeOf(Report), BytesRead);

      // extract bytes of value
      N := (Report.Bytes[1] shl 8) or Report.Bytes[2];
      // value is upper 12 bits of 16 bits
      N := (N shr 4);
      // value is 12 bit signed!
      // We need to add the upper twos complement sign bits for the Integer.
      // This is a sign extension from 12 bit to 32 bit.
      if (N and $800) <> 0 then
        N := N or Integer($FFFFF000);
      // value now is a signed integer with 2047 = 10 V and -2047 = -10 V
      Values[I].Caption := Format('%.2f', [N * 10.0 / 2048.0]);
    end;
  end;
end;

end.
