import ctypes
from ctypes import *
import time, os
import sys

#####
#####
# Sample for I2C communication with an Osram Light Sensor SFH5712 and an IO-Warrior56
#####
#####

# Available Pipes
IOW_PIPE_IO_PINS = ctypes.c_ulong(0)
IOW_PIPE_SPECIAL_MODE = ctypes.c_ulong(1)

# Product IDs
IOWKIT_PRODUCT_ID_IOW40 = 0x1500
IOWKIT_PRODUCT_ID_IOW24 = 0x1501
IOWKIT_PRODUCT_ID_IOW56 = 0x1503

# Defines
IOWKIT_MAX_PIPES = ctypes.c_ulong(2)
IOWKIT_MAX_DEVICES = ctypes.c_ulong(16)

# Count of BYTES 
IOWKIT24_IO_REPORT = ctypes.c_ubyte * 3
IOWKIT40_IO_REPORT = ctypes.c_ubyte * 5
IOWKIT_SPECIAL_REPORT = ctypes.c_ubyte * 9

IOWKIT56_IO_REPORT = ctypes.c_ubyte * 9
IOWKIT56_SPECIAL_REPORT = ctypes.c_ubyte * 65

SERIAL_NUMBER_BUFFER = ctypes.c_ubyte * 9


# Load iowkit.dll
try:
    iowkit = ctypes.WinDLL(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'iowkit'))
except:
    try:
        iowkit = ctypes.WinDLL(os.path.join(os.curdir, 'iowkit'))
    except:
        iowkit = ctypes.WinDLL('iowkit')


# Open iowkit.dll
ioHandle = iowkit.IowKitOpenDevice()

print (sys.version)

if ioHandle != 0:
    iowkit.IowKitSetTimeout(c_ulong(ioHandle), ctypes.c_ulong(1000));

    # Get number of IO-Warrior
    numdevs = iowkit.IowKitGetNumDevs()

    # For this script only one IO-Warrior
    if numdevs > 1: 
        print("Only one single IO-warrior allowed")
		iowkit.IowKitCloseDevice(ioHandle)
        exit(1)

    # Get ProductID
    pid = iowkit.IowKitGetProductId(c_ulong(ioHandle))

    # IO-Warrior56 found?
    if pid == IOWKIT_PRODUCT_ID_IOW56: print ("Device Type: IO-Warrior56")
    else: 
        print ("Only IO-Warrior56 support for these script")
        iowkit.IowKitCloseDevice(ioHandle)
        exit(1)
    
    # Get Serialnumber
    serial = create_unicode_buffer("xxxxxxxx")
    IowKit_SerNumb = windll.Iowkit.IowKitGetSerialNumber(c_ulong(ioHandle), serial) 
    print ("Serial: ", str(serial.value))
   
    print ("Enable I2C-Mode...")
   
    # Enable I2C-Mode from IO-Warrior
    report = IOWKIT56_SPECIAL_REPORT(
	                                0x01, # I2C-Mode
	                                0x01, # Enable
                                    0x00, # Flags
                                    0x00  # Timeout (only for IO-Warrior24)
                                    )
   
    # Write to IO-Warrior
    iowkit.IowKitWrite(
                       c_ulong(ioHandle),               # Device Handle
                       IOW_PIPE_SPECIAL_MODE,           # Endpoint -> for I2C: Special Mode
                       ctypes.byref(report),            # Buffer of data
                       sizeof(IOWKIT56_SPECIAL_REPORT)  # Buffersize
                       )
    
    # Init I2C-Device
    # Report struct
    report = IOWKIT56_SPECIAL_REPORT(
	                                0x02, # I2C-Write mode
	                                0xC3, # Flags : Start + Stop + 3 Byte data count
                                    0x52, # I2C-Device Address
	                                0x80, # Register
                                    0x03  # Data
                                    )
    
    iowkit.IowKitWrite(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT)) # Write to IO-Warrior
    iowkit.IowKitRead(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT)) # Read from IO-Warrior

    i = 0
    for i in range(10):    
        print ("For: ", i)
       
        # Set Register for first read position
        report = IOWKIT56_SPECIAL_REPORT(0x02, 0xC2, 0x52, 0x8C) # Write mode, Start+Stop+2 Bytes to send, I2C-Device address, Register
        iowkit.IowKitWrite(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))
        iowkit.IowKitRead(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))

        # Read Data from device
        report = IOWKIT56_SPECIAL_REPORT(0x03, 0x03, 0x53) # Read mode, 3 Bytes to read, I2C-Device address
        iowkit.IowKitWrite(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))
        iowkit.IowKitRead(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))
    
        # Output result
        print ("Output: ", (report[3] << 8) + report[2], " lux")

        # Sleeptimer
        time.sleep (250.0 / 1000.0)

    # Disable Device
    report = IOWKIT56_SPECIAL_REPORT(0x02, 0xC3, 0x52, 0x80, 0x00)
    iowkit.IowKitWrite(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))
    iowkit.IowKitRead(c_ulong(ioHandle), IOW_PIPE_SPECIAL_MODE, ctypes.byref(report), sizeof(IOWKIT56_SPECIAL_REPORT))

else:
    print('No Device')

# Don't forget to close !
iowkit.IowKitCloseDevice(ioHandle)
print ("Close iowkit, Exit")