package rc5package;
import com.codemercs.iow.*;
import java.util.Arrays;

public class Rc5Class {
	
    /** Constant for the ProductId of an IOWarrior40 */
    public static final long PID_IOW40 = 0x1500L;
    /** Constant for the ProductId of an IOWarrior24 */
    public static final long PID_IOW24 = 0x1501L;
    /** Constant for the ProductId of an IOWarrior56 */
    public static final long PID_IOW56 = 0x1503L;


	public static void main(String[] args) {
		
		long handle = IowKit.openDevice();		//Get first device Handle, which found
		int[] write = new int[8];				//Write Buffer
		String serial = IowKit.getSerialNumber(handle);	//Get serial number
		long pid = IowKit.getProductId(handle);	//Get productID
		
		if(pid == PID_IOW24) //Only IO-Warrior24 has RC5 mode
		{
			System.out.println("IO-Warrior24 found...");
			System.out.println(serial);
	
			//Enable RC5 Mode
			write[0] = 0x0C;	//Specialmode RC5
			write[1] = 0x01;	//Enable
			IowKit.write(handle,  1,  write);		//Write to endpoint 1 for specialmodes
			
			//IowKit.setTimeout(handle,  100);		//Timeout for abort readings after 50ms (only nessasary for read())
			System.out.println("exit with number 'On/Off (12)' on remote control");
			
			int[] read = new int[8];
			int laststate = 0;
			
			while(true)
			{
				//read = IowKit.read(handle,  1,  8);			//Blocks while reading no new data
				read = IowKit.readNonBlocking(handle,  1,  8);	//Don't block reading, get data immediatly
				
				if(read.length > 0)	//Data available ?
				{
					//Escape condition (12 -> default for on/off on remote control)
					if(read[1] == 12)
						break;
		
					/* Print array
					 * Index 0: ReportID
					 * Index 1: RC5-Code
					 * Index 2: Toggle bit
					 * Index Other: not used, should be 0
					 */
					
					if((read[2] != laststate) && (read[2] != 0))
					{
						System.out.println(Arrays.toString(read));
						laststate = read[2];
					}		
				}
			
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {
					System.out.println(e);
				}
			}
		}
		else
			System.out.println("IO-Warrior24 not found. Exit");
		
		System.out.println("Close and Exit");
		IowKit.closeDevice(handle);	//Dont forget to close handle and API
	}

}
