unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvHidControllerClass, IniFiles, StdCtrls;

const
  cCodeMercsVID = $07C0;
  cIOW40PID = $1500;
  cIOW24PID = $1501;

type
  TLCDReport = packed record
    ReportID: Byte;
    Flags: Byte;
    Data: array [0..5] of Byte;
  end;

  TForm1 = class(TForm)
    HidCtl: TJvHidDeviceController;
    Line0: TEdit;
    Line1: TEdit;
    Line2: TEdit;
    Line3: TEdit;
    ColumnsEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    RowsEdit: TEdit;
    Label3: TLabel;
    Clear: TButton;
    Home: TButton;
    procedure HidCtlDeviceChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure HomeClick(Sender: TObject);
  public
    NoUpdate: Boolean;
    IOWarrior: TJvHidDevice;
    Rows: Integer;
    Columns: Integer;
    Lines: array [0..3] of TEdit;
    Address: array [0..3] of Integer;
  published
    procedure InitForm(Sender: TObject);
    procedure TextChanged(Sender: TObject);
    procedure IOWEnableLCD(Enable: Boolean);
    procedure LCDWriteCmd(Cmd: Byte);
    procedure LCDClear;
    procedure LCDHome;
    procedure LCDDisplayControl(ADisplayOn, ACursorVisible, ACursorBlinks: Boolean);
    procedure LCDInit;
    procedure LCDMoveTo(Row, Column: Integer);
    procedure LCDTextOut(Line: Integer; Text: string);
  end;

var
  Form1: TForm1;

implementation

uses
  Math;

{$R *.dfm}

const
  cDefaultSection = 'Default';

procedure TForm1.IOWEnableLCD(Enable: Boolean);
var
  Report: TLCDReport;
  BytesWritten: UINT;
begin
  if Assigned(IOWarrior) then
  begin
    FillChar(Report, SizeOf(Report), 0);
    Report.ReportID := $04;
    Report.Flags := Ord(Enable);
    IOWarrior.WriteFile(Report, SizeOf(Report), BytesWritten);
  end;
end;

procedure TForm1.LCDWriteCmd(Cmd: Byte);
var
  Report: TLCDReport;
  BytesWritten: UINT;
begin
  if Assigned(IOWarrior) then
  begin
    FillChar(Report, SizeOf(Report), 0);
    Report.ReportID := $05;
    Report.Flags := $01;
    Report.Data[0] := Cmd;
    IOWarrior.WriteFile(Report, SizeOf(Report), BytesWritten);
  end;
end;

procedure TForm1.LCDClear;
begin
  LCDWriteCmd($01);
end;

procedure TForm1.LCDHome;
begin
  LCDWriteCmd($02);
end;

procedure TForm1.LCDDisplayControl(ADisplayOn, ACursorVisible, ACursorBlinks: Boolean);
begin
  LCDWriteCmd($08 or (Ord(ADisplayOn) shl 2) or (Ord(ACursorVisible) shl 1) or Ord(ACursorBlinks));
end;

procedure TForm1.LCDInit;
var
  Cmd: Byte;
begin
  // it would be possible to write 3 commands in a single report,
  // but this is easier to understand
  LCDClear;
  Cmd := $28;
  if Rows > 1 then
    Cmd := Cmd or $10;
  LCDWriteCmd(Cmd);
  LCDDisplayControl(True, True, True);
end;

procedure TForm1.LCDMoveTo(Row, Column: Integer);
begin
  case Row of
    0:
      LCDWriteCmd($80 or Column);
    1:
      LCDWriteCmd($80 or ($40 or Column));
    2:
      LCDWriteCmd($80 or ($14 or Column));
    3:
      LCDWriteCmd($80 or ($54 or Column));
  end;
end;

procedure TForm1.LCDTextOut(Line: Integer; Text: string);
var
  Report: TLCDReport;
  BytesWritten: UINT;
  I, N, Pos: Integer;
begin
  while Length(Text) < Columns do
    Text := Text + ' ';
  if Assigned(IOWarrior) then
  begin
    LCDMoveTo(Line, 0);
    Pos := 0;
    while Pos < Length(Text) do
    begin
      N := Min(6, Length(Text) - Pos);
      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $05;
      Report.Flags := $80 or N;
      for I := 0 to N - 1 do
        Report.Data[I] := Byte(Text[Pos + I + 1]);
      IOWarrior.WriteFile(Report, SizeOf(Report), BytesWritten);
      Inc(Pos, N);
    end;
  end;
end;

//===========================================================================

procedure TForm1.HidCtlDeviceChange(Sender: TObject);

  function FindIOWComplex(HidDev: TJvHidDevice): Boolean; stdcall;
  begin
    // IO-Warrior special mode interface
    Result := (HidDev.Attributes.VendorID = cCodeMercsVID) and
      ((HidDev.Attributes.ProductID = cIOW40PID) or
       (HidDev.Attributes.ProductID = cIOW24PID)) and
      (HidDev.Caps.OutputReportByteLength = 8);
  end;

begin
  // free IOW object if it has been unplugged
  if Assigned(IOWarrior) and not IOWarrior.IsPluggedIn then
    FreeAndNil(IOWarrior);
  // search for an IOW if not already found
  if not Assigned(IOWarrior) then
    if HidCtl.CheckOutByCallback(IOWarrior, @FindIOWComplex) then
    begin
      // IOW found, enable LCD mode
      IOWEnableLCD(True);
      LCDInit;
    end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  NoUpdate := True;
  Lines[0] := Line0;
  Lines[1] := Line1;
  Lines[2] := Line2;
  Lines[3] := Line3;
  with TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'LCD.INI') do
  begin
    Columns := ReadInteger(cDefaultSection, 'Columns', 16);
    Rows    := ReadInteger(cDefaultSection, 'Rows', 2);
    if Columns > 40 then
      Columns := 40;
    if Rows > 4 then
      Rows := 4;
    ColumnsEdit.Text := IntToStr(Columns);
    RowsEdit.Text := IntToStr(Rows);
    NoUpdate := False;
    InitForm(Self);
    Free;
  end;
end;

procedure TForm1.InitForm(Sender: TObject);
var
  I: Integer;
begin
  if NoUpdate then
    Exit;
  Rows := StrToIntDef(RowsEdit.Text, 2);
  Columns := StrToIntDef(ColumnsEdit.Text, 16);
  for I := Low(Lines) to High(Lines) do
  begin
    Lines[I].Visible := I < Rows;
    Lines[I].Text := Copy(Lines[I].Text, 1, Columns);
    Lines[I].MaxLength := Columns;
  end;
end;

procedure TForm1.TextChanged(Sender: TObject);
begin
  if NoUpdate then
    Exit;
  with Sender as TEdit do
    LCDTextOut(Tag, Text);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if Assigned(IOWarrior) then
  begin
    // IOW found, disable LCD mode
    LCDClear;
    LCDDisplayControl(False, False, False);
    IOWEnableLCD(False);
    FreeAndNil(IOWarrior);
  end;
  with TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'LCD.INI') do
  begin
    WriteInteger(cDefaultSection, 'Columns', Columns);
    WriteInteger(cDefaultSection, 'Rows', Rows);
    Free;
  end;
end;

procedure TForm1.ClearClick(Sender: TObject);
var
  I: Integer;
begin
  NoUpdate := True;
  for I := Low(Lines) to High(Lines) do
    Lines[I].Text := '';
  NoUpdate := False;
  LCDClear;
end;

procedure TForm1.HomeClick(Sender: TObject);
begin
  LCDHome;
end;

end.
