unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, StdCtrls, ShellAPI, ComCtrls,
  iowkit;

type
  TMatrixReport = packed record
  case Boolean of
    False:
      (ReportID: Byte;
       Row: Byte;
       Columns: DWORD;);
    True:
      (Report: IOWKIT56_SPECIAL_REPORT);
  end;

  TScroll = (stStatic, stLeft, stRight, stUp, stDown, stLeftRight, stUpDown);

  // generic component to display a matrix
  // Text display is also implemented here
  TBlinkenlichten = class(TComponent)
  private
    FColumns: Integer;
    FRows: Integer;
    FInterval: Integer;
    FScroll: TScroll;
    FText: string;
    FFont: TFont;
    FIOWarrior: IOWKIT_HANDLE;
    FIOWarriorPid: ULONG;
    FBits: array of array of Boolean;
    FBitmap: TBitmap;
    FTimer: TTimer;
    FPosition: Integer;
    FIncrement: Integer;
    FUpdateCount: Integer;
    procedure SetScroll(AScroll: TScroll);
    procedure SetRows(ARows: Integer);
    procedure SetColumns(AColumns: Integer);
    procedure SetInterval(AInterval: Integer);
    procedure SetIOWarrior(AIOWarrior: IOWKIT_HANDLE);
    function GetBits(AColumn, ARow: Integer): Boolean;
    procedure SetBits(AColumn, ARow: Integer; Value: Boolean);
    procedure TimerEvent(Sender: TObject);
    procedure ResetLength(AColumns, ARows: Integer);
    procedure SetFont(AFont: TFont);
    procedure SetText(AText: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure Update;
    procedure UpdateRows(Upper: Boolean);
    procedure UpdateRow(ARow: Integer);
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure TextOut(X, Y: Integer; Text: string);
    property IOWarrior: IOWKIT_HANDLE read FIOWarrior write SetIOWarrior;
    property IOWarriorPid: ULONG read FIOWarriorPid;
    property Bits[Column, Row: Integer]: Boolean read GetBits write SetBits;
  published
    property Columns: Integer read FColumns write SetColumns default 64;
    property Rows: Integer read FRows write SetRows default 8;
    property Interval: Integer read FInterval write SetInterval default 100;
    property Scroll: TScroll read FScroll write SetScroll default stStatic;
    property Text: string read FText write SetText;
    property Font: TFont read FFont write SetFont;
  end;

  TBlinkenMovie = class;

  TBlinkenSceneEvent = procedure(Sender: TBlinkenMovie; Scene: Integer) of object;

  // display Blinkenlights movies
  // a TBlinkenlichten is used to store and show the data
  TBlinkenMovie = class(TComponent)
  private
    FLines: TStringList;
    FBlinkenlichten: TBlinkenlichten;
    FOnScene: TBlinkenSceneEvent;
    FOnFinish: TNotifyEvent;
    FTimer: TTimer;
    FLinePos: Integer;
    FXSize: Integer;
    FYSize: Integer;
    FScene: Integer;
    procedure SetBlinkenlichten(ABlinkenlichten: TBlinkenlichten);
    procedure TimerEvent(Sender: TObject);
    function GetLines: TStrings;
    procedure NextScene;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Play;
    procedure Stop;
    procedure LoadFromFile(FileName: string);
    property Lines: TStrings read GetLines;
  published
    property Blinkenlichten: TBlinkenlichten read FBlinkenlichten write SetBlinkenlichten;
    property OnScene: TBlinkenSceneEvent read FOnScene write FOnScene;
    property OnFinish: TNotifyEvent read FOnFinish write FOnFinish;
  end;

  TCellState = (csAlive, csDead, csDying, csBorn);

  TGameOfLife = class;

  TLifeGenerationEvent = procedure(Sender: TGameOfLife; Generation: Integer) of object;

  // display Blinkenlights movies
  // a TBlinkenlichten is used to store and show the data
  TGameOfLife = class(TComponent)
  private
    FActive: Boolean;
    FColumns: Integer;
    FRows: Integer;
    FInterval: Integer;
    FCount: Integer;
    FBlinkenlichten: TBlinkenlichten;
    FOnGeneration: TLifeGenerationEvent;
    FOnFinish: TNotifyEvent;
    FCells: array of array of TCellState;
    FTimer: TTimer;
    FGeneration: Integer;
    procedure SetActive(AActive: Boolean);
    procedure SetRows(ARows: Integer);
    procedure SetColumns(AColumns: Integer);
    procedure SetInterval(AInterval: Integer);
    function GetCells(AColumn, ARow: Integer): Boolean;
    procedure SetCells(AColumn, ARow: Integer; Value: Boolean);
    procedure SetBlinkenlichten(ABlinkenlichten: TBlinkenlichten);
    procedure ResetLength(AColumns, ARows: Integer);
    procedure TimerEvent(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Clear;
    procedure Update;
    property Cells[Column, Row: Integer]: Boolean read GetCells write SetCells;
    property Count: Integer read FCount;
  published
    property Active: Boolean read FActive write SetActive default False;
    property Columns: Integer read FColumns write SetColumns default 64;
    property Rows: Integer read FRows write SetRows default 8;
    property Interval: Integer read FInterval write SetInterval default 100;
    property Blinkenlichten: TBlinkenlichten read FBlinkenlichten write SetBlinkenlichten;
    property OnGeneration: TLifeGenerationEvent read FOnGeneration write FOnGeneration;
    property OnFinish: TNotifyEvent read FOnFinish write FOnFinish;
  end;

  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    TextOutput: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    ScrollText: TComboBox;
    OpenDialog1: TOpenDialog;
    File1: TMenuItem;
    PlayGameofLife1: TMenuItem;
    PlayBlinkenMovie1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    StatusBar1: TStatusBar;
    FontDialog1: TFontDialog;
    SelectFont1: TMenuItem;
    ests1: TMenuItem;
    Speedtest1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TextOutputChange(Sender: TObject);
    procedure ScrollTextClick(Sender: TObject);
    procedure PlayGameofLife1Click(Sender: TObject);
    procedure PlayBlinkenMovie1Click(Sender: TObject);
    procedure SelectFont1Click(Sender: TObject);
    procedure Speedtest1Click(Sender: TObject);
  public
    Blinkenlichten: TBlinkenlichten;
    Life: TGameOfLife;
    Movie: TBlinkenMovie;
    PlayList: TStringList;
    IOW: IOWKIT_HANDLE;
    procedure LifeGeneration(Sender: TGameOfLife; Generation: Integer);
    procedure LifeFinish(Sender: TObject);
    procedure MovieFinish(Sender: TObject);
    procedure BlinkenScene(Sender: TBlinkenMovie; Scene: Integer);
    procedure PlayBlinkenMovie(FileName: string = '');
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  end;

var
  Form1: TForm1;

implementation

uses
  Math;

{$R *.dfm}

constructor TBlinkenlichten.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // initialize to default values
  ResetLength(64, 8);
  FScroll := stStatic;
  FIOWarrior := nil;
  FIOWarriorPid := 0;

  FUpdateCount := 0;

  // font is initialized to a reasonable default
  FFont := TFont.Create;
  FFont.Name := 'Small Fonts';
  FFont.Height := FRows;
  FFont.Color := clWhite;

  // helper bitmap to draw text
  FBitmap := TBitmap.Create;
  FBitmap.Width := FColumns;
  FBitmap.Height := FRows;
  FBitmap.PixelFormat := pf32bit;
  FBitmap.Canvas.Font := FFont;

  // timer to implement text movement
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.Interval := 100;
  FTimer.OnTimer := TimerEvent;
end;

destructor TBlinkenlichten.Destroy;
begin
  // free property variables and helpers
  FTimer.Enabled := False;
  FBitmap.Free;
  FFont.Free;
  inherited Destroy;
end;

procedure TBlinkenlichten.Clear;
var
  I, J: Integer;
begin
  // clear component matrix
  // update of the IO-Warrior is blocked to get a single update
  BeginUpdate;
  for I := 0 to Columns - 1 do
    for J := 0 to Rows - 1 do
      Bits[I, J] := False;
  FPosition := 0;
  EndUpdate;
end;

procedure TBlinkenlichten.UpdateRows(Upper: Boolean);
var
  I, J: Integer;
  Report: TMatrixReport;
  Data: array [0..31] of Byte;
begin
  if Assigned(FIOWarrior) and (FUpdateCount <= 0) then
  begin
    FillChar(Report, SizeOf(Report), 0);
    FillChar(Data[0], SizeOf(Data), 0);
    Report.ReportID := $15;
    Report.Report.Bytes[0] := Ord(Upper);
    if Upper then
    begin
      // watch the direction of the bit shift
      for J := 4 to 7 do
        for I := 0 to 63 do
          Data[(J-4)*8 + I div 8] := (Data[(J-4)*8 + I div 8] shr 1) or (Ord(Bits[I, J]) shl 7);
      Move(Data[0], Report.Report.Bytes[1], SizeOf(Data));
    end
    else
    begin
      // watch the direction of the bit shift
      for J := 0 to 3 do
        for I := 0 to 63 do
          Data[J*8 + I div 8] := (Data[J*8 + I div 8] shr 1) or (Ord(Bits[I, J]) shl 7);
      Move(Data[0], Report.Report.Bytes[1], SizeOf(Data));
    end;
    IowKitWrite(FIOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT56_SPECIAL_REPORT_SIZE);
  end;
end;

procedure TBlinkenlichten.UpdateRow(ARow: Integer);
var
  I: Integer;
  Report: TMatrixReport;
begin
  // update a single row of the IO-Warrior if update is not blocked
  if Assigned(FIOWarrior) and (FUpdateCount <= 0) and (ARow >= 0) and (ARow < 8) then
    if IOWarriorPid = IOWKIT_PID_IOW56 then
      UpdateRows(ARow >= 4)
    else
    begin
      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $15;
      Report.Row := ARow;
      Report.Columns := 0;
      // copy only max 32 bits
      // watch the direction of the bit shift
      for I := 0 to Min(Columns - 1, 31) do
        Report.Columns := (Report.Columns shr 1) or
          Cardinal(Ord(Bits[I, ARow])) shl 31;
      IowKitWrite(FIOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT_SPECIAL_REPORT_SIZE);
    end;
end;

procedure TBlinkenlichten.Update;
var
  I: Integer;
begin
  // update all rows of the IO-Warrior (max 8) if not blocked
  if Assigned(FIOWarrior) and (FUpdateCount <= 0) then
    if IOWarriorPid = IOWKIT_PID_IOW56 then
    begin
      UpdateRows(False);
      UpdateRows(True);
    end
    else
      for I := 0 to Rows - 1 do
        UpdateRow(I);
end;

procedure TBlinkenlichten.BeginUpdate;
begin
  // block update
  Inc(FUpdateCount);
end;

procedure TBlinkenlichten.EndUpdate;
begin
  // unblock update
  Dec(FUpdateCount);
  // execute update if not blocked anymore
  if FUpdateCount <= 0 then
    Update;
end;

procedure TBlinkenlichten.TextOut(X, Y: Integer; Text: string);
var
  I, J: Integer;
begin
  // write a text in the current font to the helper bitmap
  FBitmap.Canvas.Brush.Color := clBlack;
  FBitmap.Canvas.Pen.Color := clBlack;
  FBitmap.Canvas.FillRect(Rect(0, 0, Columns, Rows));
  FBitmap.Canvas.TextOut(X, Y, Text);
  // copy the helper bitmap to the matrix
  BeginUpdate;
  for I := 0 to Columns - 1 do
    for J := 0 to Rows - 1 do
      Bits[I, J] := FBitmap.Canvas.Pixels[I, J] <> clBlack;
  EndUpdate;
end;

procedure TBlinkenlichten.TimerEvent(Sender: TObject);
var
  ScrollWidth, ScrollHeight: Integer;
begin
  ScrollWidth := FBitmap.Canvas.TextWidth(Text);
  ScrollHeight := FBitmap.Canvas.TextHeight(Text);
  // calculate next text position
  case Scroll of
    stLeft:
      begin
        Dec(FPosition);
        if FPosition <= -ScrollWidth then
          FPosition := Columns;
      end;
    stRight:
      begin
        Inc(FPosition);
        if FPosition >= Columns then
          FPosition := -ScrollWidth;
      end;
    stUp:
      begin
        Dec(FPosition);
        if FPosition <= -ScrollHeight then
          FPosition := Rows;
      end;
    stDown:
      begin
        Inc(FPosition);
        if FPosition >= Rows then
          FPosition := -ScrollHeight;
      end;
    stLeftRight:
      begin
        Inc(FPosition, FIncrement);
        if (FPosition <= Min(0, Columns - ScrollWidth)) and (FIncrement < 0) then
          FIncrement := -FIncrement
        else
        if (FPosition >= Max(0, Columns - ScrollWidth)) and (FIncrement > 0) then
          FIncrement := -FIncrement;
      end;
    stUpDown:
      begin
        Inc(FPosition, FIncrement);
        if (FPosition <= Min(0, Rows - ScrollHeight)) and (FIncrement < 0) then
          FIncrement := -FIncrement
        else
        if (FPosition >= Max(0, Rows - ScrollHeight)) and (FIncrement > 0) then
          FIncrement := -FIncrement;
      end;
  end;

  // display the text (Y centered if horizontal movement for better readability)
  case Scroll of
    stLeft, stRight, stLeftRight:
      TextOut(FPosition, (Rows - Abs(Font.Height)) div 2, Text);
    stUp, stDown, stUpDown:
      TextOut(0, FPosition, Text);
  end;
end;

procedure TBlinkenlichten.SetScroll(AScroll: TScroll);
begin
  // change movement
  if AScroll <> FScroll then
  begin
    FScroll := AScroll;
    // activate timer if real movement is selected
    FTimer.Enabled := AScroll <> stStatic;
    FPosition := 0;
    FIncrement := 1;
  end;
end;

procedure TBlinkenlichten.SetInterval(AInterval: Integer);
begin
  // set speed of movement
  if AInterval > 0 then
  begin
    FInterval := AInterval;
    FTimer.Interval := AInterval;
  end;
end;

procedure TBlinkenlichten.SetRows(ARows: Integer);
begin
  if ARows <> FRows then
    ResetLength(FColumns, ARows);
end;

procedure TBlinkenlichten.SetColumns(AColumns: Integer);
begin
  if AColumns <> FColumns then
    ResetLength(AColumns, FRows);
end;

procedure TBlinkenlichten.ResetLength(AColumns, ARows: Integer);
var
  I: Integer;
begin
  for I := 0 to FColumns - 1 do
    SetLength(FBits[I], 0);
  SetLength(FBits, AColumns);
  for I := 0 to AColumns - 1 do
    SetLength(FBits[I], ARows);
  FColumns := AColumns;
  FRows := ARows;
  Clear;
end;

procedure TBlinkenlichten.SetIOWarrior(AIOWarrior: IOWKIT_HANDLE);
var
  Report: TMatrixReport;
begin
  // assign an IO-Warrior slave device for display
  if FIOWarrior <> AIOWarrior then
  begin
    FIOWarrior := AIOWarrior;
    if Assigned(AIOWarrior) then
    begin
      FIOWarriorPid := IowKitGetProductId(AIOWarrior);

      FillChar(Report, SizeOf(Report), 0);
      Report.ReportID := $14;
      Report.Report.Bytes[0] := $01;
      if IOWarriorPid = IOWKIT_PID_IOW56 then
        IowKitWrite(FIOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT56_SPECIAL_REPORT_SIZE)
      else
        IowKitWrite(FIOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT_SPECIAL_REPORT_SIZE);
    end;
    // needs update to make it show current content
    Update;
  end;
end;

function TBlinkenlichten.GetBits(AColumn, ARow: Integer): Boolean;
begin
  if (AColumn >= 0) and (AColumn < FColumns) and (ARow >= 0) and (ARow < FRows) then
    Result := FBits[AColumn][ARow]
  else
    Result := False;
end;

procedure TBlinkenlichten.SetBits(AColumn, ARow: Integer; Value: Boolean);
begin
  if (AColumn >= 0) and (AColumn < FColumns) and (ARow >= 0) and (ARow < FRows) then
    if FBits[AColumn][ARow] <> Value then
    begin
      FBits[AColumn][ARow] := Value;
      // always keep the IO-Warrior display in sync
      UpdateRow(ARow);
    end;
end;

procedure TBlinkenlichten.SetFont(AFont: TFont);
begin
  FFont.Assign(AFont);
  FBitmap.Canvas.Font := AFont;
  FPosition := 0;
  // update the text display
  TextOut(0, (Rows - Abs(Font.Height)) div 2, Text);
end;

procedure TBlinkenlichten.SetText(AText: string);
begin
  FText := AText;
  FPosition := 0;
  // update the text display
  TextOut(0, (Rows - Abs(Font.Height)) div 2, AText);
end;

//============================================================================

constructor TBlinkenMovie.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // keeps the loaded movie
  FLines := TStringList.Create;

  // timer to display the movie
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.Interval := 100;
  FTimer.OnTimer := TimerEvent;
end;

destructor TBlinkenMovie.Destroy;
begin
  Stop;
  FLines.Free;
  inherited Destroy;
end;

procedure TBlinkenMovie.LoadFromFile(FileName: string);
const
  cBlinkenLights = '# BlinkenLights Movie ';
var
  I: Integer;
  MovieLine: Integer;
  S: string;
begin
  // in case a movie is playing
  FTimer.Enabled := False;
  FLines.LoadFromFile(FileName);
  MovieLine := -1;
  // check if really a Blinkenlights movie
  for I := 0 to FLines.Count - 1 do
    if Pos(cBlinkenLights, FLines[I]) > 0 then
    begin
      MovieLine := I;
      Break;
    end;
  if MovieLine = -1 then
    raise Exception.Create('not a BlinkenLights Movie');
  // extract width and height of movie
  I := Pos(cBlinkenLights, Lines[MovieLine]) + Length(cBlinkenLights);
  S := Copy(Lines[MovieLine], I, Length(Lines[MovieLine]));
  I := Pos('x', S);
  FXSize := StrToInt(Copy(S, 1, I - 1));
  FYSize := StrToInt(Copy(S, I + 1, Length(S)));
  // initialize/rewind to first line
  FLinePos := 0;
end;

procedure TBlinkenMovie.NextScene;
var
  I, J: Integer;
  S: string;
begin
  // stop timer in case there is no further scene
  FTimer.Enabled := False;
  // check for end of file
  if FLinePos >= FLines.Count then
  begin
    Stop;
    if Assigned(FOnFinish) then
      FOnFinish(Self);
    Exit;
  end;
  // ignore empty lines and comment lines
  S := Trim(FLines[FLinePos]);
  while (S = '') or (S[1] = '#') do
  begin
    Inc(FLinePos);
    // check for end of file
    if FLinePos >= FLines.Count then
      Break;
    S := Trim(FLines[FLinePos]);
  end;
  // check for end of file
  if FLinePos >= FLines.Count then
  begin
    Stop;
    if Assigned(FOnFinish) then
      FOnFinish(Self);
    Exit;
  end;
  // check for scene
  if S[1] = '@' then
  begin
    // get time to display scene
    FTimer.Interval := StrToInt(Copy(S, 2, Length(S)));
    Inc(FLinePos);
    Blinkenlichten.BeginUpdate;
    for I := 0 to FYSize - 1 do
    begin
      // check for premature end of file
      if FLinePos >= FLines.Count then
        Break;
      S := Trim(FLines[FLinePos]);
      if S <> '' then
        for J := 0 to FXSize - 1 do
           Blinkenlichten.Bits[J, I] := S[J+1] = '1';
      Inc(FLinePos);
    end;
    Blinkenlichten.EndUpdate;
    Inc(FScene);
    if Assigned(FOnScene) then
      FOnScene(Self, FScene);
    FTimer.Enabled := True;
  end
  else
    FLinePos := FLines.Count;
end;

procedure TBlinkenMovie.TimerEvent(Sender: TObject);
begin
  NextScene;
end;

procedure TBlinkenMovie.Play;
begin
  if Assigned(FBlinkenlichten) then
  begin
    Stop; // in case a movie is already playing
    FLinePos := 0; // Rewind
    FScene := 0;
    NextScene;
  end;
end;

procedure TBlinkenMovie.Stop;
begin
  FTimer.Enabled := False;
  // stop implies clear of slave control
  if Assigned(FBlinkenlichten) then
    FBlinkenlichten.Clear;
end;

function TBlinkenMovie.GetLines: TStrings;
begin
  Result := FLines;
end;

procedure TBlinkenMovie.SetBlinkenlichten(ABlinkenlichten: TBlinkenlichten);
begin
  if FBlinkenlichten <> ABlinkenlichten then
  begin
    FBlinkenlichten := ABlinkenlichten;
    // initialize slave control by clearing it
    if Assigned(ABlinkenlichten) then
      ABlinkenlichten.Clear;
  end;
end;

//============================================================================

constructor TGameOfLife.Create(AOwner: TComponent);
begin
  // initialize to default values
  inherited Create(AOwner);
  ResetLength(64, 8);
  FBlinkenlichten := nil;
  FCount := 0;

  // timer to play game
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FTimer.Interval := 100;
  FTimer.OnTimer := TimerEvent;
end;

destructor TGameOfLife.Destroy;
begin
  FTimer.Enabled := False;
  inherited Destroy;
end;

procedure TGameOfLife.Clear;
var
  I, J: Integer;
begin
  if Assigned(FBlinkenlichten) then
    FBlinkenlichten.BeginUpdate;
  for I := 0 to Columns - 1 do
    for J := 0 to Rows - 1 do
      Cells[I, J] := False;
  if Assigned(FBlinkenlichten) then
    FBlinkenlichten.EndUpdate;
end;

procedure TGameOfLife.Update;
var
  I, J: Integer;
begin
  if Assigned(FBlinkenlichten) then
  begin
    FBlinkenlichten.BeginUpdate;
    for I := 0 to Columns - 1 do
      for J := 0 to Rows - 1 do
        FBlinkenlichten.Bits[I, J] := Cells[I, J];
    FBlinkenlichten.EndUpdate;
  end;
end;

procedure TGameOfLife.TimerEvent(Sender: TObject);

  procedure ApplyRules;
  var
    I, J: Integer;

    function TestNeighborCells(X, Y: Integer): Integer;
    var
      XM, YM, XP, YP: Integer;
    begin
      XM := X - 1;
      YM := Y - 1;
      XP := X + 1;
      YP := Y + 1;

      if X = 0 then
        XM := Columns - 1;
      if X = Columns - 1 then
        XP := 0;
      if Y = 0 then
        YM := Rows - 1;
      if Y = Rows - 1 then
        YP := 0;

      Result := 0;
      if FCells[XM][YM] in [csAlive, csDying] then
        Inc(Result);
      if FCells[X][YM] in [csAlive, csDying] then
        Inc(Result);
      if FCells[XP][YM] in [csAlive, csDying] then
        Inc(Result);
      if FCells[XP][Y] in [csAlive, csDying] then
        Inc(Result);
      if FCells[XP][YP] in [csAlive, csDying] then
        Inc(Result);
      if FCells[X][YP] in [csAlive, csDying] then
        Inc(Result);
      if FCells[XM][YP] in [csAlive, csDying] then
        Inc(Result);
      if FCells[XM][Y] in [csAlive, csDying] then
        Inc(Result);
    end;

  begin
    for I := 0 to Columns - 1 do
      for J := 0 to Rows - 1 do
        case TestNeighborCells(I, J) of
          0, 1, 4, 5, 6, 7, 8:
            if FCells[I][J] = csAlive then
              FCells[I][J] := csDying;
          3:
            if FCells[I][J] = csDead then
              FCells[I][J] := csBorn;
        end;
  end;

  procedure NewGeneration;
  var
    I, J: Integer;
  begin
    FCount := 0;
    for I := 0 to FColumns - 1 do
      for J := 0 to FRows - 1 do
      begin
        if FCells[I][J] = csDying then
          FCells[I][J] := csDead
        else
        if FCells[I][J] = csBorn then
          FCells[I][J] := csAlive;
        Inc(FCount, Ord(FCells[I][J] = csAlive));
      end;
  end;

begin
  ApplyRules;
  NewGeneration;
  Inc(FGeneration);
  Update;
  if Assigned(FOnGeneration) then
    FOnGeneration(Self, FGeneration);
  // autodeactivate if extinct
  if Count = 0 then
  begin
    if Assigned(FOnFinish) then
      FOnFinish(Self);
    Active := False;
  end;
end;

procedure TGameOfLife.SetActive(AActive: Boolean);
begin
  FActive := AActive;
  FTimer.Enabled := AActive;
  FGeneration := 0;
end;

procedure TGameOfLife.SetRows(ARows: Integer);
begin
  if ARows <> FRows then
    ResetLength(FColumns, ARows);
end;

procedure TGameOfLife.SetColumns(AColumns: Integer);
begin
  if AColumns <> FColumns then
    ResetLength(AColumns, FRows);
end;

procedure TGameOfLife.ResetLength(AColumns, ARows: Integer);
var
  I: Integer;
begin
  for I := 0 to FColumns - 1 do
    SetLength(FCells[I], 0);
  SetLength(FCells, AColumns);
  for I := 0 to AColumns - 1 do
    SetLength(FCells[I], ARows);
  FColumns := AColumns;
  FRows := ARows;
  Clear;
end;

procedure TGameOfLife.SetInterval(AInterval: Integer);
begin
  if AInterval > 0 then
  begin
    FInterval := AInterval;
    FTimer.Interval := AInterval;
  end;
end;

function TGameOfLife.GetCells(AColumn, ARow: Integer): Boolean;
begin
  if (AColumn >= 0) and (AColumn < FColumns) and (ARow >= 0) and (ARow < FRows) then
    Result := FCells[AColumn][ARow] = csAlive
  else
    Result := False;
end;

procedure TGameOfLife.SetCells(AColumn, ARow: Integer; Value: Boolean);
begin
  if (AColumn >= 0) and (AColumn < FColumns) and (ARow >= 0) and (ARow < FRows) then
  begin
    if Value then
    begin
      if FCells[AColumn][ARow] <> csAlive then
      begin
        Inc(FCount);
        FCells[AColumn][ARow] := csAlive;
        if Assigned(FBlinkenlichten) then
          FBlinkenlichten.Bits[AColumn, ARow] := True;
      end;
    end
    else
    begin
      if FCells[AColumn][ARow] <> csDead then
      begin
        Dec(FCount);
        FCells[AColumn][ARow] := csDead;
        if Assigned(FBlinkenlichten) then
          FBlinkenlichten.Bits[AColumn, ARow] := False;
      end;
    end;
  end;
end;

procedure TGameOfLife.SetBlinkenlichten(ABlinkenlichten: TBlinkenlichten);
begin
  if FBlinkenlichten <> ABlinkenlichten then
  begin
    FBlinkenlichten := ABlinkenlichten;
    // reset the size of the matrix of cells to the size of the slave control matrix
    if Assigned(ABlinkenlichten) then
      ResetLength(ABlinkenlichten.Columns, ABlinkenlichten.Rows);
  end;
end;

//============================================================================

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  LoadIowKitAPI;
  Blinkenlichten := TBlinkenlichten.Create(Self);
  PlayList := TStringList.Create;
  Life := TGameOfLife.Create(Self);
  Life.OnGeneration := LifeGeneration;
  Life.OnFinish := LifeFinish;
  Movie := TBlinkenMovie.Create(Self);
  Movie.OnScene := BlinkenScene;
  Movie.OnFinish := MovieFinish;
  DragAcceptFiles(Handle, True);
  for I := 1 to ParamCount do
  begin
    if LowerCase(ExtractFileExt(ParamStr(I))) = '.blm' then
      PlayList.Add(ParamStr(I));
  end;

  Blinkenlichten.IOWarrior := IowKitOpenDevice;
  PlayBlinkenMovie;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Report: TMatrixReport;
begin
  DragAcceptFiles(Handle, False);
  if Assigned(Blinkenlichten.IOWarrior) then
  begin
    // disable matrix mode
    FillChar(Report, SizeOf(Report), 0);
    Report.ReportID := $14;
    // Report.Bytes[0] := $00;
    if Blinkenlichten.IOWarriorPid = IOWKIT_PID_IOW56 then
      IowKitWrite(Blinkenlichten.IOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT56_SPECIAL_REPORT_SIZE)
    else
      IowKitWrite(Blinkenlichten.IOWarrior, IOW_PIPE_SPECIAL_MODE, PChar(@Report.Report), IOWKIT_SPECIAL_REPORT_SIZE);
    Blinkenlichten.IOWarrior := nil;
  end;
  PlayList.Free;
  IowKitCloseDevice(Blinkenlichten.IOWarrior);
  UnloadIowKitAPI;
end;

procedure TForm1.WMDropFiles(var Msg: TWMDropFiles);
var
  I, N: Integer;
  Buffer: array [0..1023] of Char;
begin
  N := DragQueryFile(Msg.Drop, $FFFFFFFF, Buffer, SizeOf(Buffer));
  for I := 0 to N - 1 do
  begin
    DragQueryFile(Msg.Drop, I, Buffer, SizeOf(Buffer));
    if LowerCase(ExtractFileExt(Buffer)) = '.blm' then
      PlayList.Add(Buffer);
  end;
  DragFinish(Msg.Drop);
  PlayBlinkenMovie;
  Msg.Result := 0;
end;

procedure TForm1.LifeGeneration(Sender: TGameOfLife; Generation: Integer);
begin
  StatusBar1.SimpleText := Format('Generation %d', [Generation]);
end;

procedure TForm1.LifeFinish(Sender: TObject);
begin
  PlayGameofLife1.Checked := False;
end;

procedure TForm1.BlinkenScene(Sender: TBlinkenMovie; Scene: Integer);
begin
  StatusBar1.SimpleText := Format('Scene %d', [Scene]);
end;

procedure TForm1.MovieFinish(Sender: TObject);
begin
  PlayBlinkenMovie;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.TextOutputChange(Sender: TObject);
begin
  PlayGameofLife1.Checked := False;
  Life.Active := False;
  Movie.Stop;
  Blinkenlichten.Text := TextOutput.Text;
  Blinkenlichten.Scroll := TScroll(ScrollText.ItemIndex);
end;

procedure TForm1.ScrollTextClick(Sender: TObject);
begin
  Life.Active := False;
  Movie.Stop;
  Blinkenlichten.Text := TextOutput.Text;
  Blinkenlichten.Scroll := TScroll(ScrollText.ItemIndex);
end;

procedure TForm1.PlayGameofLife1Click(Sender: TObject);
var
  I, J: Integer;
begin
  if PlayGameofLife1.Checked then
  begin
    // stop any activity
    Life.Active := False;
    Movie.Stop;
    Blinkenlichten.Text := '';
    Life.Blinkenlichten := Blinkenlichten;
    // initialize to random pattern (about 1/3 of cells on)
    Life.Blinkenlichten.BeginUpdate;
    Randomize;
    for I := 0 to Life.Columns - 1 do
      for J := 0 to Life.Rows - 1 do
        Life.Cells[I, J] := Random(2) = 0;
    Life.Blinkenlichten.EndUpdate;
    Life.Active := True;
  end
  else
  begin
    Life.Active := False;
    Life.Clear;
  end;
end;

procedure TForm1.PlayBlinkenMovie1Click(Sender: TObject);
var
  I: Integer;
begin
  if OpenDialog1.Execute then
  begin
    for I := 0 to OpenDialog1.Files.Count - 1 do
      PlayList.Add(OpenDialog1.Files[I]);
    PlayBlinkenMovie;
  end;
end;

procedure TForm1.SelectFont1Click(Sender: TObject);
begin
  FontDialog1.Font := Blinkenlichten.Font;
  if FontDialog1.Execute then
    Blinkenlichten.Font := FontDialog1.Font;
end;

procedure TForm1.PlayBlinkenMovie(FileName: string);
begin
  // stop any activity
  PlayGameofLife1.Checked := False;
  Life.Active := False;
  Movie.Stop;
  Blinkenlichten.Text := '';
  Movie.Blinkenlichten := Blinkenlichten;
  // add movie to playlist
  if FileName <> '' then
    PlayList.Add(FileName);
  // play next entry in playlist
  if PlayList.Count > 0 then
  begin
    Movie.LoadFromFile(PlayList[0]);
    PlayList.Delete(0);
    Movie.Play;
  end;
end;

procedure TForm1.Speedtest1Click(Sender: TObject);
var
  I, J, N: Integer;
  Start, Stop: DWORD;
begin
  I := 0;
  J := 0;
  Blinkenlichten.Clear;
  Start := GetTickCount;
  for N := 0 to Blinkenlichten.Columns*Blinkenlichten.Rows - 1 do
  begin
    Blinkenlichten.Bits[I, J] := True;
    Inc(I);
    if I >= Blinkenlichten.Columns then
    begin
      I := 0;
      Inc(J);
    end;
  end;
  Stop := GetTickCount;
  StatusBar1.SimpleText := Format('%u msecs', [Stop - Start]);
end;

end.
