
// PWM-GeneratorDlg.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "PWM-Generator.h"
#include "PWM-GeneratorDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CPWMGeneratorDlg-Dialogfeld



CPWMGeneratorDlg::CPWMGeneratorDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(IDD_PWMGENERATOR_DIALOG, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CPWMGeneratorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CMB_DEVICE, m_Device);
	DDX_Control(pDX, IDC_CMB_CLOCK, m_Clock);
	DDX_Control(pDX, IDC_EDIT_PERIODE_MSB, m_Period);
	DDX_Control(pDX, IDC_EDIT_PULSE_MSB, m_Pulse);
}

BEGIN_MESSAGE_MAP(CPWMGeneratorDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_BTN_SEARCH, &CPWMGeneratorDlg::OnBnClickedBtnSearch)
	ON_CBN_SELCHANGE(IDC_CMB_DEVICE, &CPWMGeneratorDlg::OnCbnSelchangeCmbDevice)
	ON_BN_CLICKED(IDC_BTN_SET, &CPWMGeneratorDlg::OnBnClickedBtnSet)
END_MESSAGE_MAP()


// CPWMGeneratorDlg-Meldungshandler

BOOL CPWMGeneratorDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden

	m_Clock.SetCurSel(0);
	m_Period.SetLimitText(5);	//Max digits for 16 Bit
	m_Pulse.SetLimitText(5);	//Max digits for 16 Bit

	//Set some default values
	m_Period.SetWindowTextW(L"512");
	m_Pulse.SetWindowTextW(L"128");

	return TRUE; 
}


void CPWMGeneratorDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // Gertekontext zum Zeichnen

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Symbol in Clientrechteck zentrieren
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Symbol zeichnen
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

HCURSOR CPWMGeneratorDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



void CPWMGeneratorDlg::OnClose()
{
	if (m_Handle != NULL)
	{
		//Disable PWM mode
		IOWKIT56_SPECIAL_REPORT report;
		memset(&report, 0x00, IOWKIT56_SPECIAL_REPORT_SIZE);

		//Create report
		report.ReportID = 0x20;			//Special mod PWM output
		report.Bytes[0] = 0x00;			//Enable

		//Send report to IO-Warrior56
		IowKitWrite(m_SelectedHandle, IOW_PIPE_SPECIAL_MODE, (char*)&report, IOWKIT56_SPECIAL_REPORT_SIZE);

		//Dont forget to close !!!!
		IowKitCloseDevice(m_Handle);
	}
		
	CDialogEx::OnClose();
}

void CPWMGeneratorDlg::OnBnClickedBtnSearch()
{
	IOWKIT_HANDLE tHandle = NULL;	//temp handle
	CString text;
	UINT pid;
	ULONG rev;
	int index = 0;	//index for m_Devices -> Only IOW56 supports PWM

	//Open iowkit API
	m_Handle = NULL;
	m_Handle = IowKitOpenDevice();

	m_Device.ResetContent();
		
	//One or more devices connected and iowkit not open by another application
	if (m_Handle != NULL)
	{
		//Get count of iows
		ULONG numDev = IowKitGetNumDevs();
		
		//Get unique handles for each connected iow
		for (ULONG i = 1; i <= numDev; i++)
		{
			//get handle
			tHandle = IowKitGetDeviceHandle(i);
			
			//Get product ID from tHandle
			pid = IowKitGetProductId(tHandle);
			rev = IowKitGetRevision(tHandle);

			TraceInt(rev, L"Rev: ");

			//Get revision HEX from String
			long l = strtol("2000", NULL, 16);

			TraceInt(l, L"Rev: ");

			//Only IOW56 rev 2.0.0.0 supports PWM output
			if ((pid == IOWKIT_PID_IOW56) && (l <= rev))
			{
				m_Devices[index] = tHandle;
				text = GetProductStringExt(m_Devices[index]);
				m_Device.AddString(text);

				index++; //Count the IOW56 with rev >= 2.0.0.0
			}
		}

		if (index == 1)
		{
			m_Device.SetCurSel(0);
			m_SelectedHandle = m_Devices[0];
			IowKitSetTimeout(m_Devices[0], 2000);
			GetDlgItem(IDC_BTN_SET)->EnableWindow(TRUE);
			
		}
		if (index == 0)
		{
			MessageBox(L"No IO-Warrior56 with rev. >= 2.0.0.0 found on USB.", L"Device error", MB_OK | MB_ICONERROR);
		}
	}
	else
	{
		MessageBox(L"No IO-Warrior found on USB.", L"Device error", MB_OK | MB_ICONERROR);
	}
}


void CPWMGeneratorDlg::OnCbnSelchangeCmbDevice()
{
	m_SelectedHandle = m_Devices[m_Device.GetCurSel()];
	GetDlgItem(IDC_BTN_SET)->EnableWindow(TRUE);
}


void CPWMGeneratorDlg::OnBnClickedBtnSet()
{
	CString value;
	BYTE period_msb = 0x00;
	BYTE period_lsb = 0x00;
	BYTE pulse_msb = 0x00;
	BYTE pulse_lsb = 0x00;
	BYTE clock = 0;

	IOWKIT56_SPECIAL_REPORT report;
	memset(&report, 0x00, IOWKIT56_SPECIAL_REPORT_SIZE);


	clock = m_Clock.GetCurSel();

	if ((m_Period.GetIntValue() <= 0xFFFF) && (m_Pulse.GetIntValue() <= 0xFFFF) && (m_Period.GetWindowTextLengthW() > 0) && (m_Pulse.GetWindowTextLengthW()))
	{
		if (m_Pulse.GetIntValue() >= m_Period.GetIntValue())
			MessageBox(L"Pulse value should not be greater than or equal to period value.", L"Information", MB_OK | MB_ICONINFORMATION);

		//Get MSB and LSB from input
		period_msb = (m_Period.GetIntValue() & 0xFF00) >> 8;
		period_lsb = (m_Period.GetIntValue() & 0x00FF);

		pulse_msb = (m_Pulse.GetIntValue() & 0xFF00) >> 8;
		pulse_lsb = (m_Pulse.GetIntValue() & 0x00FF);


		//Create report
		report.ReportID = 0x20;			//Special mod PWM output
		report.Bytes[0] = 0x01;			//Enable
		report.Bytes[1] = period_lsb;	//period LSB
		report.Bytes[2] = period_msb;	//period MSB
		report.Bytes[3] = pulse_lsb;	//pulse LSB
		report.Bytes[4] = pulse_msb;	//pulse MSB
		report.Bytes[5] = clock;		//Clock

		//Send report to IO-Warrior56
		IowKitWrite(m_SelectedHandle, IOW_PIPE_SPECIAL_MODE, (char*)&report, IOWKIT56_SPECIAL_REPORT_SIZE);
		//No IowKitRead() nessesary!
	}
	else
	{
		MessageBox(L"Period or pulse are too high or empty.", L"Error", MB_OK | MB_ICONERROR);
	}
}