Option Strict Off
Option Explicit On
Friend Class Form1
	Inherits System.Windows.Forms.Form
	' This is Basic IO-Warrior usage sample
	' This sample shows how to program IO-Warrior for input and output
	' and how to use multiple devices
	
	' IO-Warrior handles
	Dim iowHandles(IOWKIT_MAX_DEVICES) As Integer
	' Number of IOWs
	Dim numIows As Integer
	' Data buffer
	' Basic data buffer is 8 bytes long: 1 byte for report id, max 7 bytes for data
	Dim data(8) As Byte
	
	Private Sub WriteLEDs(ByRef Value As Byte)
		' Number of bytes written
		Dim nWritten As Integer
		' Loop variable
		Dim I As Integer
		' ProductID of IOWarrior
		Dim Pid As Integer
		
		' Set up Report ID
		data(0) = 0
		For I = 0 To numIows - 1
			' Set up bytes
			data(1) = &HFFs
			data(2) = &HFFs
			data(3) = &HFFs
			data(4) = &HFFs
			data(5) = &HFFs
			data(6) = &HFFs
			data(7) = &HFFs
			
			Pid = IowKitGetProductId(iowHandles(I))
			If Pid = IOWKIT_PID_IOW24 Then
				' First byte is controlling standard LED
				data(1) = Value
				' Write to IOW 24
				nWritten = IowKitWrite(iowHandles(I), IOW_PIPE_IO_PINS, data(0), 3)
			Else
				If Pid = IOWKIT_PID_IOW40 Then
					' Fourth byte is controlling standard LEDs
					data(4) = Value
					' Write to IOW 40
					nWritten = IowKitWrite(iowHandles(I), IOW_PIPE_IO_PINS, data(0), 5)
				Else
					If Pid = IOWKIT_PID_IOW56 Then
						' Seventh byte is controlling standard LEDs
						data(7) = Value
						' Write to IOW 56
						nWritten = IowKitWrite(iowHandles(I), IOW_PIPE_IO_PINS, data(0), 8)
					End If
				End If
			End If
		Next I
	End Sub
	
	Private Sub ClearAllButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles ClearAllButton.Click
		' Clear all LEDs (negative logic)
		Call WriteLEDs(&HFFs)
	End Sub
	
	' Blink all IOWs
	Private Sub RandomButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles RandomButton.Click
		' Write random bits to LEDs
		Randomize()
		Call WriteLEDs(Int(256 * Rnd()) And &HFFs)
	End Sub
	
	Private Sub Form1_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
		Dim I As Integer
		Dim S(18) As Byte
		
		' Initialize everything
		' Open device
		iowHandles(0) = IowKitOpenDevice()
		' Fail if can't open
		If iowHandles(0) = 0 Then
			' Barf and exit from program
			MsgBox("Can not open device!", 0, "Error")
			End
		End If
		' Get number of IO-Warrior devices in system
		numIows = IowKitGetNumDevs()
		' Get all IO-Warrior handles
		For I = 2 To numIows
			iowHandles(I - 1) = IowKitGetDeviceHandle(I)
		Next I
		Call ClearAllButton_Click(ClearAllButton, New System.EventArgs())
		I = IowKitGetSerialNumber(iowHandles(0), S(0))
		ReadLabel.Text = System.Text.UnicodeEncoding.Unicode.GetString(S)
		ReadLabel.Text = "Serial number: " & ReadLabel.Text
		ReadLabel.Refresh()
	End Sub
	
	Private Sub Form1_FormClosed(ByVal eventSender As System.Object, ByVal eventArgs As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
		Call ClearAllButton_Click(ClearAllButton, New System.EventArgs())
		
		' Close all IO-Warrior devices
		IowKitCloseDevice((iowHandles(0)))
	End Sub
	
	' Read something from IO-Warrior
	Private Sub ReadButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles ReadButton.Click
		Dim Res As Integer
		Dim N As Integer
		Dim Pid As Integer
		
		' Perform a read
		Pid = IowKitGetProductId(iowHandles(0))
		If Pid = IOWKIT_PID_IOW24 Then
			N = 3
		Else
			If Pid = IOWKIT_PID_IOW40 Then
				N = 5
			Else
				If Pid = IOWKIT_PID_IOW56 Then
					N = 8
				End If
			End If
		End If
		
		ReadLabel.Text = "Reading from IOW"
		ReadLabel.Refresh()
		' Read from IO-Warrior
		Res = IowKitRead(iowHandles(0), 0, data(0), N)
		' Check results
		If Res <> N Then
			ReadLabel.Text = "Read from IOW wrong?"
		Else
			' Convert and set data
			TextVal3.Text = ""
			TextVal4.Text = ""
			TextVal5.Text = ""
			TextVal6.Text = ""
			TextVal7.Text = ""
			
			TextVal1.Text = Hex(data(1))
			TextVal2.Text = Hex(data(2))
			If N = 5 Or N = 8 Then
				TextVal3.Text = Hex(data(3))
				TextVal4.Text = Hex(data(4))
			End If
			If N = 8 Then
				TextVal5.Text = Hex(data(5))
				TextVal6.Text = Hex(data(6))
				TextVal7.Text = Hex(data(7))
			End If
			ReadLabel.Text = "Read from IOW complete"
		End If
	End Sub
	
	' Write value
	Private Sub WriteButton_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles WriteButton.Click
		Dim Res As Integer
		Dim Value As Byte
		Dim N As Integer
		Dim Pid As Integer
		
		Pid = IowKitGetProductId(iowHandles(0))
		If Pid = IOWKIT_PID_IOW24 Then
			N = 3
		Else
			If Pid = IOWKIT_PID_IOW40 Then
				N = 5
			Else
				If Pid = IOWKIT_PID_IOW56 Then
					N = 8
				End If
			End If
		End If
		
		' Report ID 0
		data(0) = 0
		' Get values from text
		' Note: we don't check values here
		' so you'd better enter correct values
		Value = Val("&H" & TextVal1.Text)
		data(1) = Value
		Value = Val("&H" & TextVal2.Text)
		data(2) = Value
		If N = 5 Or N = 8 Then
			Value = Val("&H" & TextVal3.Text)
			data(3) = Value
			Value = Val("&H" & TextVal4.Text)
			data(4) = Value
		End If
		If N = 8 Then
			Value = Val("&H" & TextVal5.Text)
			data(5) = Value
			Value = Val("&H" & TextVal6.Text)
			data(6) = Value
			Value = Val("&H" & TextVal7.Text)
			data(7) = Value
		End If
		
		' Write to IOW
		Res = IowKitWrite(iowHandles(0), 0, data(0), N)
		' Handle error
		If Res <> N Then
			' You can get error code from GetLastError()
			MsgBox("Write to IOW failed!", 0, "Error")
		End If
	End Sub
End Class