unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvHidControllerClass, StdCtrls, ExtCtrls, TeEngine,
  Series, TeeProcs, Chart, JvComponentBase;

const
  cCodeMercsVID = $07C0;
  cIOW24PID = $1501;
  cIOW56PID = $1503;

type
  TSPIReport = packed record
    ReportID: Byte;
    case Boolean of
    False:
     (Length: Byte;
      Value: array [0..1] of Byte;);
    True:
      (Bytes: array [0..63] of Byte;);
  end;

  TForm1 = class(TForm)
    HidCtl: TJvHidDeviceController;
    Temperature: TLabel;
    Kicker: TTimer;
    Chart1: TChart;
    Series1: TLineSeries;
    procedure HidCtlDeviceChange(Sender: TObject);
    procedure HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
      const Data: Pointer; Size: Word);
    procedure FormDestroy(Sender: TObject);
    procedure KickerTimer(Sender: TObject);
  public
    IOWSPI: TJvHIDDevice;
    Counter: Integer;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.HidCtlDeviceChange(Sender: TObject);
var
  Report: TSPIReport;
  BytesWritten: DWORD;

  function FindIOW24Complex(HidDev: TJvHidDevice): Boolean; stdcall;
  begin
    // IO-Warrior 24, 56 special mode interface match criteria
    Result := (HidDev.Attributes.VendorID = cCodeMercsVID) and
      (((HidDev.Attributes.ProductID = cIOW24PID) and
        (HidDev.Caps.OutputReportByteLength = 8)) or
       ((HidDev.Attributes.ProductID = cIOW56PID) and
        (HidDev.Caps.OutputReportByteLength = 64)));
  end;

begin
  // free IOW object if it has been unplugged
  if Assigned(IOWSPI) and not IOWSPI.IsPluggedIn then
  begin
    // disable reader timer
    Kicker.Enabled := False;
    FreeAndNil(IOWSPI);
  end;
  // search for an IOW if not already found
  if not Assigned(IOWSPI) then
    if HidCtl.CheckOutByCallback(IOWSPI, @FindIOW24Complex) then
    begin
      if IOWSPI.Attributes.ProductID = cIOW24PID then
      begin
        // IOW24 found, enable SPI
        FillChar(Report, SizeOf(Report), 0);
        Report.ReportID := $08;
        Report.Bytes[0] := $01; // enable SPI
        Report.Bytes[1] := $07; // mode /CPOL, CPHA, 62.5 KBit
        // the chip can even be contacted at 2 MBit, but this may cause
        // occasional corrupt data
        IOWSPI.WriteFile(Report, IOWSPI.Caps.OutputReportByteLength, BytesWritten);
      end
      else
      begin
        // IOW56 found, enable SPI
        FillChar(Report, SizeOf(Report), 0);
        Report.ReportID := $08; //SPI-Mode
        Report.Bytes[0] := $01; //Enable SPI-Mode
        Report.Bytes[1] := $01; // mode MSB first, /CPOL, CPHA
        Report.Bytes[2] := $00; // 93.75 KBit
        // the chip can even be contacted at 2 MBit, but this may cause
        // occasional corrupt data
        IOWSPI.WriteFile(Report, IOWSPI.Caps.OutputReportByteLength, BytesWritten);
      end;
      // init chart
      Chart1.Series[0].Clear;
      Counter := 0;
      // enable reader timer
      Kicker.Enabled := True;
    end
    else
      // no IOW24, show bogus temperature
      Temperature.Caption := '?,?? C';
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Report: TSPIReport;
  BytesWritten: DWORD;
begin
  // if IOW still plugged when program terminates
  if Assigned(IOWSPI) then
  begin
    // disable SPI
    FillChar(Report, SizeOf(Report), 0);
    Report.ReportID := $08;
    // Report.Bytes[0] := $00; // disable SPI
    IOWSPI.WriteFile(Report, IOWSPI.Caps.OutputReportByteLength, BytesWritten);
  end;
end;

procedure TForm1.KickerTimer(Sender: TObject);
var
  Report: TSPIReport;
  BytesWritten: DWORD;
begin
  // SPI transactions are always read/write pairs
  // write two bytes to SPI to trigger read of two bytes
  FillChar(Report, SizeOf(Report), 0);
  Report.ReportID := $09; // write SPI
  Report.Bytes[0] := $02; // two dummy bytes
  Report.Bytes[1] := $00; //flags for IOW56
  IOWSPI.WriteFile(Report, IOWSPI.Caps.OutputReportByteLength, BytesWritten);
end;

procedure TForm1.HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
  const Data: Pointer; Size: Word);
var
  Val: WORD;
  Report: TSPIReport;
begin
  // read the two bytes triggered by KickerTimer
  if ReportID = $09 then
  begin
    // extract temperature value
    Move(Data^, Report.Bytes[0], 3);
    // Bit 2 is high if the chip has no contact to thermo sensor
    // so we use the value only if bit is low
    if (Report.Value[1] and $4) = 0 then
    begin
      Val := (Word(Report.Value[0]) shl 8) or Word(Report.Value[1]);
      // lower 3 bits are not part of the temperature value
      Val := Val shr 3;
      // temperature is in 1/4 degree Celsius
      Temperature.Caption := Format('%f C', [Val / 4]);
      // display last 60 values in chart
      if Chart1.Series[0].Count > 60 then
        Chart1.Series[0].Delete(0);
      Chart1.Series[0].AddXY(Counter, Val / 4);
    end;
    Inc(Counter);
  end;
end;

end.
