#pragma once

#include "stdafx.h"
#include "iow28_adc.h"

void SetParameter(IOWKIT_HANDLE handle, BYTE enable, BYTE chan, BYTE mod)
{
	IOWKIT28_SPECIAL_REPORT report;
	memset(&report, 0, IOWKIT28_ADC_REPORT_SIZE);
	
	report.ReportID = IOW28_ADC_MODE;
	report.Bytes[0] = enable;		//Enable / Disable ADC mode
	report.Bytes[1] = chan;			//Channel count 
	report.Bytes[2] = 0x00;			//Must be 0
	report.Bytes[3] = 0x00;			//Must be 0
	report.Bytes[4] = mod;			//Single short or continues mode

	IowKitWrite(handle, IOW_PIPE_ADC_MODE, (char*)&report, IOWKIT28_ADC_REPORT_SIZE);
}

IOW28_ADC_CALIB_STRUCT GetCalibrationData(IOWKIT_HANDLE handle)
{
	IOW28_ADC_CALIB_STRUCT data = { 0 };
	IOWKIT28_SPECIAL_REPORT report;
	
	memset(&report, 0, IOWKIT28_ADC_REPORT_SIZE);
	report.ReportID = IOW28_ADC_DATA;
	report.Bytes[0] = IOW28_ADC_CALIBRATION;

	if (IowKitWrite(handle, IOW_PIPE_ADC_MODE, (char*)&report, IOWKIT28_ADC_REPORT_SIZE) == IOWKIT28_ADC_REPORT_SIZE)
	{
		if (IowKitRead(handle, IOW_PIPE_ADC_MODE, (char*)&report, IOWKIT28_ADC_REPORT_SIZE) == IOWKIT28_ADC_REPORT_SIZE)
		{
			data.VrefCal = (report.Bytes[2] << 8) | report.Bytes[1];
			data.TempCal1 = (report.Bytes[4] << 8) | report.Bytes[3];
			data.TempCal2 = (report.Bytes[6] << 8) | report.Bytes[5];
			data.Vref = (report.Bytes[8] << 8) | report.Bytes[7];
			data.Temp = (report.Bytes[10] << 8) | report.Bytes[9];
		}
		else
			AfxMessageBox(L"IowKitRead() ERROR", MB_OK | MB_ICONSTOP);
	}
	else
		AfxMessageBox(L"IowKitWrite() ERROR", MB_OK | MB_ICONSTOP);

	return data;
}

IOW28_ADC_DATA_STRUCT GetADC_SingleShot(IOWKIT_HANDLE handle)
{
	IOW28_ADC_DATA_STRUCT data;
	IOWKIT28_SPECIAL_REPORT report;
	
	memset(&report, 0, IOWKIT28_ADC_REPORT_SIZE);
	report.ReportID = IOW28_ADC_DATA;
	report.Bytes[0] = IOW28_ADC_VALUES;

	if (IowKitWrite(handle, IOW_PIPE_ADC_MODE, (char*)&report, IOWKIT28_ADC_REPORT_SIZE) == IOWKIT28_ADC_REPORT_SIZE)
	{
		if (IowKitRead(handle, IOW_PIPE_ADC_MODE, (char*)&report, IOWKIT28_ADC_REPORT_SIZE) == IOWKIT28_ADC_REPORT_SIZE)
		{
			data.seq = report.Bytes[0];
			data.Ch1 = (report.Bytes[2] << 8) | report.Bytes[1];
			data.Ch2 = (report.Bytes[4] << 8) | report.Bytes[3];
			data.Ch3 = (report.Bytes[6] << 8) | report.Bytes[5];
			data.Ch4 = (report.Bytes[8] << 8) | report.Bytes[7];
		}
		else
			AfxMessageBox(L"IowKitRead() ERROR", MB_OK | MB_ICONSTOP);
	}
	else
		AfxMessageBox(L"IowKitWrite() ERROR", MB_OK | MB_ICONSTOP);

	return data;
}

/*Get ADC value as mili voltage*/
double GetVoltage(unsigned short VrefCal, unsigned short Vref, unsigned short ADCdata)
{
	return 3.3/*V*/ * VrefCal * ADCdata / Vref / 4095;
}

/*Get temperature from sensor */
double GetTemperatur(unsigned short TempCal1, unsigned short TempCal2, unsigned short VrefCal, unsigned short Vref, unsigned short Temp)
{
	return 80.0/*C*/ / (TempCal2 - TempCal1) * (Temp * VrefCal / Vref - TempCal1) + 30.0;
}