#include <windows.h>
#include <stdio.h>
#include <iostream>
#include "iowkit.h"

void EnableI2c(IOWKIT_HANDLE handle, BYTE status, BYTE flags, BYTE timeout)
{
   IOWKIT28_SPECIAL_REPORT report;
   memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);

   report.ReportID = 0x01;    //Special mode: I2C
   report.Bytes[0] = status;  //0 or 1
   report.Bytes[1] = flags;
   report.Bytes[2] = timeout;

   IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE);
   //No IowKitRead() required!
}

int main(int argc, char** argv)
{
   IOWKIT_HANDLE handle;
   IOWKIT28_SPECIAL_REPORT report;

   memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
   handle = IowKitOpenDevice();

   //Go on if a device was found
   if(handle != NULL)
   {
      if(IowKitGetNumDevs() != 1)
      {
         printf("Sample only works with on connected IO-Warrior28. Exit\n");
         IowKitCloseDevice(handle); //Never forget to close on applicaion ends!
         return 0;
      }

      if(IowKitGetProductId(handle) != IOWKIT_PID_IOW28)
      {
         printf("Sample only works with IO-Warrior28. Exit\n");
         IowKitCloseDevice(handle); //Never forget to close on applicaion ends!
         return 0;
      }

      printf("\n\nInterface open\n");
      printf("\n\nSearch for I2C devices...\n");
      printf("=========================\n");

      //Enable I2C mode
      EnableI2c(handle, 0x01, 0x00, 0x00);

      //Search for I2C devices by reading 1 byte
      for(int i=1; i<127; i++)
      {
         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x03;
         report.Bytes[0] = 0x01;
         report.Bytes[1] = (i << 1) | 0x01;
         report.Bytes[2] = 0x00;

         if(IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
         {
            if(IowKitRead(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
            {
               if(report.Bytes[0] == 0x01) //Read byte 0, if byte count = 1 -> device present
                  printf("I2C device found (8Bit): 0x%02X\n", (i << 1));
            }
            else
            {
               printf("ERROR on IowKitRead(). CANCEL\n");
               break;
            }
         }
         else
            {
               printf("ERROR on IowKitWrite(). CANCEL\n");
               break;
            }
      }
      //Disable I2C mode
      EnableI2c(handle, 0x00, 0x00, 0x00);
      
   }
   else
      printf("No IO-Warrior found!\n");



   IowKitCloseDevice(handle);
   printf("=========================\n");
   printf("Over and out!\n\n\n");

   return 0; 
}