//Sample for IO-Warrior28 and I2C mode.
//Read data from I2C device

#include <windows.h>
#include <stdio.h>
#include <iostream>
#include "iowkit.h"

#define I2C_ENABLE 0x01
#define I2C_DISBALE 0x00

void EnableI2c(IOWKIT_HANDLE handle, BYTE status, BYTE flags, BYTE timeout)
{
   IOWKIT28_SPECIAL_REPORT report;
   memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);

   report.ReportID = 0x01;    //Special mode: I2C
   report.Bytes[0] = status;  //0 or 1
   report.Bytes[1] = flags;
   report.Bytes[2] = timeout;

   IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE);
   //No IowKitRead() required!
}

int WriteData(IOWKIT_HANDLE handle, IOWKIT28_SPECIAL_REPORT rep)
{
   if(IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&rep, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
   {
      if(IowKitRead(handle, IOW_PIPE_I2C_MODE, (char*)&rep, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
      {
         if((rep.Bytes[0] & 0x80) == 0x80) //Error flag
            printf("Error flag found, errorcode: 0x%02X\n", rep.Bytes[1]);
         else
            printf("Write successful\n");
      }
      else
         printf("ERROR on IowKitRead(). CANCEL\n");
   }
   else
      printf("ERROR on IowKitWrite(). CANCEL\n");

   return rep.Bytes[0]; //return flags
}

int main(int argc, char** argv)
{
   IOWKIT_HANDLE handle;
   IOWKIT28_SPECIAL_REPORT report;

   //Open iowkit and get first IO-Warrior on USB which found
   handle = IowKitOpenDevice();

   //Go on if a device was found
   if(handle != NULL)
   {
      if(IowKitGetProductId(handle) == IOWKIT_PID_IOW28)
      {
         //Enable I2C mode
         EnableI2c(handle, I2C_ENABLE, 0, 0);

         //Set start register on I2C device to read from (EEPROM Atmel AT32C32)
         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x02;    //I2C write mode
         report.Bytes[0] = 0xC3;    //Flags: Startbit + Stopbit, byte count (incl. i2c address)
         report.Bytes[1] = 0xA0;    //I2C 8Bit device address
         report.Bytes[2] = 0x00;    //Data
         report.Bytes[3] = 0x00;    //Data
         WriteData(handle, report);

         //Read data from device
         int count = 15;
         int count_lsb = count & 0xFF;
         int count_msb = (count & 0xFF00) >> 8;


         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x03;             //I2C read mode
         report.Bytes[0] = count_lsb;        //Data count LSB
         report.Bytes[1] = 0xA0 | 0x01;      //I2C 8Bit device address + read bit
         report.Bytes[2] = count_msb;        //Data Count MSB (if more than 254 in LSB)
         //Send read request
         if(IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
         {
            //Read data (if less than 63 byte, for more read X times)
            if(IowKitRead(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
            {
               if((report.Bytes[0] & 0x80) == 0x80) //Error flag
                  printf("Error flag found, errorcode: 0x%02X\n", report.Bytes[1]);
               else //Everything is fine, show data
               {
                  for(int i=0; i<count+1; i++)
                  {
                     printf("Byte[%d]: 0x%02X\n", i, report.Bytes[i]);
                  }
               }
            }
            else
               printf("ERROR on IowKitRead(). CANCEL\n");
         }
         else
            printf("ERROR on IowKitWrite(). CANCEL\n");


         //Disable I2C mode
         EnableI2c(handle, I2C_DISBALE, 0, 0);
      }
      else
         printf("No IO-Warrior28 found! Exit\n");
   }
   else
      printf("No IO-Warrior found!\n");

   //Never forget to close iowkit on exit application!
   IowKitCloseDevice(handle);

   return 0; 
}