//Sample for IO-Warrior28 and I2C mode.
//Writing I2C data in individual reports

#include <windows.h>
#include <stdio.h>
#include <iostream>
#include "iowkit.h"

#define I2C_ENABLE 0x01
#define I2C_DISBALE 0x00

void EnableI2c(IOWKIT_HANDLE handle, BYTE status, BYTE flags, BYTE timeout)
{
   IOWKIT28_SPECIAL_REPORT report;
   memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);

   report.ReportID = 0x01;    //Special mode: I2C
   report.Bytes[0] = status;  //0 or 1
   report.Bytes[1] = flags;
   report.Bytes[2] = timeout;

   IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT28_SPECIAL_REPORT_SIZE);
   //No IowKitRead() required!
}

int WriteData(IOWKIT_HANDLE handle, IOWKIT28_SPECIAL_REPORT rep)
{
   if(IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&rep, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
   {
      if(IowKitRead(handle, IOW_PIPE_I2C_MODE, (char*)&rep, IOWKIT28_SPECIAL_REPORT_SIZE) == IOWKIT28_SPECIAL_REPORT_SIZE)
      {
         if((rep.Bytes[0] & 0x80) == 0x80) //Error flag
            printf("Error flag found, errorcode: 0x%02X\n", rep.Bytes[1]);
         else
            printf("Write successful\n");
      }
      else
         printf("ERROR on IowKitRead(). CANCEL\n");
   }
   else
      printf("ERROR on IowKitWrite(). CANCEL\n");

   return rep.Bytes[0]; //return flags
}

int main(int argc, char** argv)
{
   IOWKIT_HANDLE handle;
   IOWKIT28_SPECIAL_REPORT report;

   //Open iowkit and get first IO-Warrior on USB which found
   handle = IowKitOpenDevice();

   //Go on if a device was found
   if(handle != NULL)
   {
      if(IowKitGetProductId(handle) == IOWKIT_PID_IOW28)
      {
         //Enable I2C mode
         EnableI2c(handle, I2C_ENABLE, 0, 0);

         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x02;    //I2C write mode
         report.Bytes[0] = 0xC2;    //Flags: Startbit + Stopbit, byte count (incl. i2c address)
         report.Bytes[1] = 0xE0;    //I2C 8Bit device address
         report.Bytes[2] = 0x21;    //Data

         WriteData(handle, report);

         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x02;    //I2C write mode
         report.Bytes[0] = 0xC2;    //Flags: Startbit + Stopbit, byte count (incl. i2c address)
         report.Bytes[1] = 0xE0;    //I2C 8Bit device address
         report.Bytes[2] = 0x81;    //Data

         WriteData(handle, report);

         memset(&report, 0, IOWKIT28_SPECIAL_REPORT_SIZE);
         report.ReportID = 0x02;    //I2C write mode
         report.Bytes[0] = 0xC3;    //Flags: Startbit + Stopbit, byte count (incl. i2c address)
         report.Bytes[1] = 0xE0;    //I2C 8Bit device address
         report.Bytes[2] = 0x00;    //Data
         report.Bytes[3] = 0x5A;    //Data

         WriteData(handle, report);

         //Disable I2C mode
         EnableI2c(handle, I2C_DISBALE, 0, 0);
      }
      else
         printf("No IO-Warrior28 found! Exit\n");
   }
   else
      printf("No IO-Warrior found!\n");

   //Never forget to close iowkit on exit application!
   IowKitCloseDevice(handle);

   return 0; 
}